package com.ibm.oti.shared;

import java.security.BasicPermission;
import java.security.Permission;
import java.security.PermissionCollection;
import java.util.StringTokenizer;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 1998, 2010  All Rights Reserved
 */

/**
 * Security permission to govern classloader access to the shared class cache.
 * <p>
 * Usage: To grant permission to a classloader, add permission in java.policy file.<br>
 * Eg. com.ibm.oti.shared.SharedClassPermission "classloaders.myClassLoader", "read,write";
 * <p>
 * "read" allows a classloader to load classes from the shared cache<br>
 * "write" allows a classloader to add classes to the shared cache
 * <p>
 * @author		OTI
 * @version		initial
 */
public class SharedClassPermission extends BasicPermission {

	private static final long serialVersionUID = -3867544018716468265L;

	transient private boolean read, write;

	/**
	 * Constructs a new instance of this class.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 * @param		loader java.lang.ClassLoader
	 *					the ClassLoader requiring the permission.
	 * @param		actions java.lang.String
	 *					the actions which are applicable to it.
	 */
	public SharedClassPermission(ClassLoader loader, String actions) {
		this(loader.getClass().getName(), actions);
	}

	/**
	 * Constructs a new instance of this class.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 * @param		classLoaderClassName java.lang.String
	 *					the className of the ClassLoader requiring the permission.
	 * @param		actions java.lang.String
	 *					the actions which are applicable to it.
	 */
	public SharedClassPermission(String classLoaderClassName, String actions) {
		super(classLoaderClassName);
		decodeActions(actions);
	}

	private void decodeActions(String actions) {
		StringTokenizer tokenizer =
			new StringTokenizer(actions.toLowerCase(), " \t\n\r,");
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken();
			if (token.equals("read"))
				read = true;
			else if (token.equals("write"))
				write = true;
			else
				throw new IllegalArgumentException();
		}
		if (!read && !write) throw new IllegalArgumentException();
	}

	/**
	 * Compares the argument to the receiver, and answers true
	 * if they represent the <em>same</em> object using a class
	 * specific comparison. In this case, the receiver must be
	 * for the same property as the argument, and must have the
	 * same actions.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @param		o		the object to compare with this object
	 * @return		<code>true</code>
	 *					if the object is the same as this object
	 *				<code>false</code>
	 *					if it is different from this object
	 * @see			#hashCode
	 */
	public boolean equals(Object o) {
		if (super.equals(o)) {
			SharedClassPermission pp = (SharedClassPermission) o;
			return (read == pp.read) && (write == pp.write);
		}
		return false;
	}

	/**
	 * Answers a new PermissionCollection for holding permissions
	 * of this class. Answer null if any permission collection can
	 * be used.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @return		a new PermissionCollection or null
	 *
	 * @see			java.security.BasicPermissionCollection
	 */
	public PermissionCollection newPermissionCollection() {
		return new SharedClassPermissionCollection();
	}

	/**
	 * Answers an integer hash code for the receiver. Any two
	 * objects which answer <code>true</code> when passed to
	 * <code>equals</code> must answer the same value for this
	 * method.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @return		the receiver's hash
	 *
	 * @see			#equals
	 */
	public int hashCode() {
		return super.hashCode();
	}

	/**
	 * Answers the actions associated with the receiver.
	 * The result will be either "read", "write", or
	 * "read,write".
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @return		String
	 *					the actions associated with the receiver.
	 */
	public String getActions() {
		return read ? (write ? "read,write" : "read") : "write";
	}

/*  Truth table for implies method:
 *
 * 	property				implies
 *	p1 read		p1 write 	p2 read		p2 write	result
 *	0			0			0			0			0
 *	0			0			0			1			0
 *	0			0			1			0			0
 *	0			0			1			1			0
 *	0			1			0			0			1
 *	0			1			0			1			1
 *	0			1			1			0			0
 *	0			1			1			1			0
 *	1			0			0			0			1
 *	1			0			0			1			0
 *	1			0			1			0			1
 *	1			0			1			1			0
 *	1			1			0			0			1
 *	1			1			0			1			1
 *	1			1			1			0			1
 *	1			1			1			1			1
 */
	/**
	 * Indicates whether the argument permission is implied
	 * by the receiver.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @return		boolean
	 *					<code>true</code> if the argument permission
	 *					is implied by the receiver,
	 *					and <code>false</code> if it is not.
	 * @param		permission java.security.Permission
	 *					the permission to check
	 */
	public boolean implies(Permission permission) {
		if (super.implies(permission)) {
			SharedClassPermission pp = (SharedClassPermission) permission;
			boolean result = (read && write) ||
							(write && !pp.read) ||
							(read && !pp.write);
			return result;
		}
		return false;
	}
}
